( cd usr/lib ; rm -rf libfaad.so.0 )
( cd usr/lib ; ln -sf libfaad.so.0.0.0 libfaad.so.0 )
( cd usr/lib ; rm -rf libfaad.so )
( cd usr/lib ; ln -sf libfaad.so.0.0.0 libfaad.so )
( cd usr/lib ; rm -rf libmp4v2.so.0 )
( cd usr/lib ; ln -sf libmp4v2.so.0.0.0 libmp4v2.so.0 )
( cd usr/lib ; rm -rf libmp4v2.so )
( cd usr/lib ; ln -sf libmp4v2.so.0.0.0 libmp4v2.so )
( cd usr/lib ; rm -rf libmp4ff.so.0 )
( cd usr/lib ; ln -sf libmp4ff.so.0.0.0 libmp4ff.so.0 )
( cd usr/lib ; rm -rf libmp4ff.so )
( cd usr/lib ; ln -sf libmp4ff.so.0.0.0 libmp4ff.so )
